package org.average.nfcauthcr;

import java.lang.String;
import java.io.IOException;
import java.util.Arrays;

import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.TagLostException;
import android.nfc.tech.IsoDep;

import org.average.nfcauthcr.CRException;

public class YkNeo {

	// This is a CCID APDU, ISO 7816-4.
	// 00 A4 04 00 xx AID - GlobalPlatform - SELECT
	// Lc, send data = 07: A0 00 00 05 27 20 01
	// Le, recv data = 00
	private static final byte[] selectApdu =
		{0x00, (byte) 0xA4, 0x04, 0x00, 0x07, (byte) 0xA0,
		 0x00, 0x00, 0x05, 0x27, 0x20, 0x01, 0x00};

	private static final byte SLOT_CHAL_HMAC1 = 0x30;
	private static final byte SLOT_CHAL_HMAC2 = 0x38;

	public static final byte[] doChallengeYubiKey(IsoDep isoTag, int slot,
						byte[] challenge)
				throws CRException {
		try {
			isoTag.connect();
			byte[] resp = isoTag.transceive(selectApdu);
			int length = resp.length;
			if (resp[length - 2] != (byte)0x90 ||
			    resp[length - 1] != 0x00) {
				throw new CRException(String.format(
				"NFC select error code: %02x:%02x",
				resp[length - 2], resp[length - 1]));
			}
			byte[] crApdu = new byte[69];
			crApdu[0] = 0x00; // CLA
			crApdu[1] = 0x01; // INS
			switch (slot) {
			case 1: crApdu[2] = SLOT_CHAL_HMAC1; break; // P1
			case 2: crApdu[2] = SLOT_CHAL_HMAC2; break; // P1
			}
			crApdu[3] = 0x00; // P2
			crApdu[4] = 63;   // Lc
			System.arraycopy(challenge, 0, crApdu, 5,
						challenge.length); // Payload
			crApdu[crApdu.length-1] = 22; // Le
			resp = isoTag.transceive(crApdu);
			length = resp.length;
			if (resp[length - 2] != (byte)0x90 ||
			    resp[length - 1] != 0x00) {
				throw new CRException(String.format(
				"NFC CR error code: %02x:%02x",
				resp[length - 2], resp[length - 1]));
			}
			if (length != 22) {
				throw new CRException(String.format(
				"NFC wrong response size: got %d, need 20",
				length-2));
			}
			return Arrays.copyOf(resp, length-2);
		} catch (TagLostException e) {
			throw new CRException("NFC connection lost", e);
		} catch (IOException e) {
			throw new CRException("NFC I/O: " + e.getMessage(), e);
		}
	}
}
